@@ -3,7 +3,7 @@
 #include "image.h"
 
 static const nv_image_t bnc_cable = {
-  64, 64, 4, 0,
+  64, 64, 4,
   "\224\372\372\372\0\6\371\371\371\0\366\366\366\0\265\265\265\0\356\356\356"
   "\0\273\273\273\0\261\260\261\0\202\332\332\332\0\6\311\311\311\0\225\224"
   "\225\0\256\253\256\0\315\311\315\0\304\302\304\0\216\215\216\0\214\0\0\0"
